TOOL_ELEMENTS = {
    "search_the_whole_bible": "search:::",
    "compare_bible_translations": "verses:::KJV,LEB,NET,OHGB,OHGBi:::",
    "retrieve_bible_cross_references": "xrefs:::",
    "retrieve_hebrew_or_greek_bible_verses": "verses:::OHGB:::",
    "retrieve_interlinear_hebrew_or_greek_bible_verses": "verses:::OHGBi:::",
    "retrieve_bible_verses": "verses:::",
    "retrieve_verse_morphology": "morphology:::",
    "retrieve_bible_chapter": "chapter:::",
    "read_bible_commentary": "commentary:::",
    "refine_bible_translation": {'system': 'bible/translate'},
    "write_pastor_prayer": {'system': 'bible/pray'},
    "ask_theologian": {'system': 'bible/theologian'},
    "quote_bible_verses": {'system': 'bible/quote'},
    "anyalyze_psalms": {'system': 'bible/david'},
    "ask_pastor": {'system': 'bible/billy'},
    "ask_bible_scholar": {'system': 'bible/scholar'},
    "write_bible_perspectives": {'system': 'bible/perspective'},
    "explain_bible_meaning": {'instruction': 'bible/meaning', 'system': 'auto'},
    "write_new_testament_historical_context": {'instruction': 'bible/nt_context', 'system': 'auto'},
    "write_bible_questions": {'instruction': 'bible/questions', 'system': 'auto'},
    "write_bible_devotion": {'instruction': 'bible/devotion', 'system': 'auto'},
    "translate_hebrew_bible_verse": {'instruction': 'bible/translate_hebrew', 'system': 'auto'},
    "write_bible_location_study": {'instruction': 'bible/location', 'system': 'auto'},
    "translate_greek_bible_verse": {'instruction': 'bible/translate_greek', 'system': 'auto'},
    "identify_bible_keywords": {'instruction': 'bible/keywords', 'system': 'auto'},
    "study_old_testament_themes": {'instruction': 'bible/ot_themes', 'system': 'auto'},
    "study_new_testament_themes": {'instruction': 'bible/nt_themes', 'system': 'auto'},
    "write_old_testament_highlights": {'instruction': 'bible/ot_highligths', 'system': 'auto'},
    "write_bible_prayer": {'instruction': 'bible/prayer', 'system': 'auto'},
    "write_short_bible_prayer": {'instruction': 'bible/short_prayer', 'system': 'auto'},
    "write_bible_character_study": {'instruction': 'bible/character', 'system': 'auto'},
    "write_bible_thought_progression": {'instruction': 'bible/flow', 'system': 'auto'},
    "quote_bible_promises": {'instruction': 'bible/promises', 'system': 'auto'},
    "write_bible_chapter_summary": {'instruction': 'bible/chapter_summary', 'system': 'auto'},
    "interpret_old_testament_verse": {'instruction': 'bible/ot_meaning', 'system': 'auto'},
    "expound_bible_topic": {'instruction': 'bible/topic', 'system': 'auto'},
    "write_bible_theology": {'instruction': 'bible/theology', 'system': 'auto'},
    "study_bible_themes": {'instruction': 'bible/themes', 'system': 'auto'},
    "write_bible_canonical_context": {'instruction': 'bible/canon', 'system': 'auto'},
    "write_bible_related_summary": {'instruction': 'bible/summary', 'system': 'auto'},
    "interpret_new_testament_verse": {'instruction': 'bible/nt_meaning', 'system': 'auto'},
    "write_new_testament_highlights": {'instruction': 'bible/nt_highlights', 'system': 'auto'},
    "write_bible_applications": {'instruction': 'bible/application', 'system': 'auto'},
    "write_bible_book_introduction": {'instruction': 'bible/introduce_book', 'system': 'auto'},
    "write_old_testament_historical_context": {'instruction': 'bible/ot_context', 'system': 'auto'},
    "write_bible_outline": {'instruction': 'bible/outline', 'system': 'auto'},
    "write_bible_insights": {'instruction': 'bible/insights', 'system': 'auto'},
    "write_bible_sermon": {'instruction': 'bible/sermon', 'system': 'auto'},
}